package com.ezet.crowdstream.client.services

import android.util.Log
import io.reactivex.subjects.PublishSubject
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStreamReader
import java.io.PrintWriter
import java.net.InetAddress
import java.net.Socket
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors
import java.util.concurrent.Future

class TcpClient {

    private val executorService: ExecutorService = Executors.newFixedThreadPool(2)
    val data: PublishSubject<String> = PublishSubject.create()
    val events: PublishSubject<Int> = PublishSubject.create()
    private var serverIp: String = ""
    private var serverPort: Int = 0
    var autoConnect = true
    private var run = true
    private var bufferOut: PrintWriter? = null
    private var bufferIn: BufferedReader? = null
    private var socket: Socket? = null
    private var task: Future<*>? = null

    fun connect(ip: String, port: Int) {
        if (task != null) {
            disconnect()
        }
        this.serverIp = ip
        this.serverPort = port
        run = true
        task = executorService.submit(::run)

    }

    fun sendMessage(message: String) {
        Log.d(TAG, "sendMessage: $message")
        if (bufferOut != null && !bufferOut!!.checkError()) {
            bufferOut?.print(message + delimiter)
            bufferOut?.flush()
        }
    }

    fun disconnect() {
        Log.d(TAG, "disconnect() called")
        run = false
        bufferOut?.let { it.flush(); it.close() }
        if (socket != null) {
            try {
                socket?.close()
            } catch (e: IOException) {
                e.printStackTrace()
            }

        }
        bufferIn = null
        bufferOut = null
        task?.cancel(true)
        task = null
    }

    private fun run() {
        var remainingAttempts = Companion.MAX_ATTEMPTS
        while (autoConnect and run && remainingAttempts > 0) {
            try {
                events.onNext(CONNECTING)
                val serverAddr = InetAddress.getByName(serverIp)
                Log.i(TAG, "Connecting...")
                Socket(serverAddr, serverPort).use { socket ->
                    this.socket = socket
                    events.onNext(CONNECTED)
                    bufferOut = PrintWriter(socket.getOutputStream())
                    Log.i(TAG, "Connected")
                    bufferIn = BufferedReader(InputStreamReader(socket.getInputStream()))
                    while (run) {
                        val msg = bufferIn!!.readLine()
                        msg?.let { data.onNext(it) }
                    }
                    events.onNext(DISCONNECTED)
                    data.onComplete()
                    events.onComplete()
                    socket.close()
                }
            } catch (e: Exception) {
                Log.e(TAG, "", e)
                events.onNext(DISCONNECTED)
                --remainingAttempts
            }
        }
    }

    companion object {
        private val TAG = TcpClient::class.java.name
        private val delimiter = "\r\n"
        val CONNECTING: Int = 0
        val CONNECTED: Int = 1
        val DISCONNECTED: Int = 2
        private var MAX_ATTEMPTS = 10
    }
}
