package com.ezet.crowdstream.client.streamactivity.tasks

import android.os.AsyncTask
import android.os.SystemClock
import android.util.Log

import com.ezet.crowdstream.client.streamactivity.StreamSurfaceView

import java.lang.ref.WeakReference

class DisplaySequenceTask(view: StreamSurfaceView, private val targetDelay: Long) : AsyncTask<String, Void, Void>() {

    private var sleep: Long = 0
    private var lastUpdate: Long = 0
    private var totalError: Long = 0
    private val view: WeakReference<StreamSurfaceView> = WeakReference(view)

    init {
        this.sleep = targetDelay
    }

    override fun doInBackground(vararg strings: String): Void? {
        for (symbol in strings[0].toCharArray()) {
            val streamView = view.get() ?: return null
            when (symbol) {
                '1' -> streamView.drawColor(255, 255, 255)
                else -> streamView.drawColor(0, 0, 0)
            }
            Log.d(Companion.TAG, "doInBackground: sleep:" + sleep)
            SystemClock.sleep(sleep)

            val currentTime = SystemClock.elapsedRealtime()
            if (lastUpdate != 0L) {
                val actualDelay = currentTime - lastUpdate
                val correction = targetDelay - actualDelay
                totalError += correction
                sleep = Math.max(targetDelay + totalError, 0)
                Log.d(Companion.TAG, "doInBackground: actual delay: " + actualDelay)
            }
            lastUpdate = currentTime
        }
        view.get()?.drawColor(0, 0, 0)
        Log.d(Companion.TAG, "doInBackground: total error: " + totalError)
        return null
    }

    companion object {
        private val TAG = "DisplaySequenceTask"
    }
}
