import logging
import sys

from twisted.internet import reactor
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet.protocol import connectionDone
from twisted.protocols.basic import LineOnlyReceiver

logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
logger = logging.getLogger(__name__)

DEBUG = False


class AdminClient(LineOnlyReceiver):
    def __init__(self):
        self.ip = None
        self.port = None
        self.password = None
        # noinspection PyProtectedMember
        from twisted.internet.selectreactor import SelectReactor
        self.reactor: SelectReactor = reactor

    def lineReceived(self, line: bytes):
        print(line.decode())
        self.reactor.callFromThread(self.read_input)

    def send(self, data: str):
        self.reactor.callFromThread(self.sendLine, data.strip().encode())

    def read_connection_lost(self):
        logger.info('read_connection_lost')
        self.disconnect()

    def connectionLost(self, reason=connectionDone):
        logger.info("connection lost: " + reason.getErrorMessage())
        self.reactor.stop()

    def disconnect(self):
        self.transport.loseConnection()

    def connectionMade(self):
        print('connected')
        self.send(f"auth {self.password}")

    def connect(self, ip, port, password):
        self.ip = ip
        self.port = port
        self.password = password
        endpoint = TCP4ClientEndpoint(reactor, ip, port)
        connectProtocol(endpoint, self)
        print('connecting...')
        # TODO: timeout
        self.reactor.run()

    def _get_prompt_string(self):
        return f"{self.ip}:{self.port}> "

    def read_input(self):
        data = input(self._get_prompt_string()).strip()
        if data in ['q', 'exit', 'quit']:
            self.disconnect()
        self.sendLine(data.encode())


if __name__ == "__main__":
    AdminClient().connect('127.0.0.1', 8000, '123123')
