import logging
from threading import Condition

from twisted.internet import reactor
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet.protocol import connectionDone
from twisted.protocols.basic import LineOnlyReceiver

# logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
logger = logging.getLogger(__name__)

DEBUG = False


class CliClient(LineOnlyReceiver):

    @staticmethod
    def send(ip, port, password, data):
        client = CliClient(ip, port, password, data)
        client.connect()

    def __init__(self, ip, port, password, data):
        self.ip = ip
        self.port = port
        self.password = password
        self.data = data
        # noinspection PyProtectedMember
        from twisted.internet.selectreactor import SelectReactor
        self.reactor: SelectReactor = reactor
        self.exit = False
        self.cv = Condition()

    def lineReceived(self, data: bytes):
        line = data.decode()
        if line == 'authentication successful':
            self._send(self.data)
        elif line == 'ok':
            self.disconnect()
            with self.cv:
                self.cv.notify()
        else:
            print(line)

    def _send(self, data: str):
        self.reactor.callFromThread(self.sendLine, data.strip().encode())

    def read_connection_lost(self):
        logger.info('read_connection_lost')
        self.disconnect()

    def connectionLost(self, reason=connectionDone):
        self.reactor.stop()

    def disconnect(self):
        self.transport.loseConnection()
        self.exit = True

    def connectionMade(self):
        self._send(f"auth {self.password}")

    def connect(self):
        endpoint = TCP4ClientEndpoint(reactor, self.ip, self.port)
        from twisted.internet.defer import Deferred
        deferred: Deferred = connectProtocol(endpoint, self)
        deferred.addBoth(lambda: print('cb'))
        deferred.addTimeout(timeout=4, clock=self.reactor)
        self.reactor.run()
        with self.cv:
            result = self.cv.wait_for(lambda: self.exit, 5)
        if not result:
            deferred.cancel()
            print('Connection timed out')
        else:
            deferred.cancel()
