import random
import sys
from typing import List, Tuple

import cv2
import numpy as np


def find_nearest_neighbour(position: (int, int), points: List[Tuple[int, int]]) -> (int, int):
    nearest = -1
    (x, y) = position
    min_distance = sys.maxsize
    for i, (tx, ty) in enumerate(points):
        distance = abs(x - tx) + abs(y - ty)
        if distance < min_distance:
            nearest = i
            min_distance = distance
    return min_distance, nearest


def create_empty_frame(width, height):
    return np.zeros((height, width, 3), dtype='uint8')


def generate_sequence(length, prefix, streak_limit):
    sequence = list(prefix)
    streak = 1
    for _ in range(length - len(prefix)):
        if streak == streak_limit:
            symbol = '1' if sequence[-1] == '0' else '0'
            streak = 1
        else:
            symbol = str(random.randint(0, 1))
            if symbol == sequence[-1]:
                streak += 1
        sequence.append(symbol)
    return "".join(sequence)


def draw_circle(img, center, color=(0, 255, 0)):
    cv2.circle(img, center, 5, color, 2)


def milliseconds():
    import time
    return int(round(time.time() * 1000))


COLOR_RED = (0, 0, 255)
COLOR_YELLOW = (0, 255, 255)
COLOR_GREEN = (0, 255, 0)
COLOR_BLUE = (255, 0, 0)
COLOR_MAGENTA = (255, 255, 0)
