from typing import List, Tuple

import cv2

from crowdstream.utils.tracking import COLOR_TRACK_FOUND, COLOR_TRACK_LOST


def debug_tracking(frames, timestamps, tracks, debug_frame_delay=10):
    for frame_idx, frame in enumerate(frames):
        for track in tracks:
            for state_idx, (visible, position, timestamp, state) in enumerate(track.history):
                if timestamp == timestamps[frame_idx]:
                    if state:
                        if state_idx == 0 or not track.history[state_idx - 1].state:
                            cv2.circle(img=frame, center=position, radius=4, color=COLOR_TRACK_FOUND, thickness=1,
                                       lineType=cv2.LINE_AA)
                        else:
                            cv2.circle(img=frame, center=position, radius=4, color=(255, 255, 0), thickness=1,
                                       lineType=cv2.LINE_AA)
                    elif not state and track.history[state_idx - 1].state:
                        cv2.circle(img=frame, center=position, radius=4, color=COLOR_TRACK_LOST, thickness=1,
                                   lineType=cv2.LINE_AA)
        cv2.imshow('tracking', frame)
        cv2.waitKey(debug_frame_delay)
    cv2.destroyWindow('tracking')


def draw_tracking(frame, detections, color):
    for point in detections:
        cv2.circle(img=frame, center=point, radius=4, color=color, thickness=1, lineType=cv2.LINE_AA)


def debug_detections(frames, detections: List[List[Tuple[int, int]]], debug_frame_delay=10):
    for frame_idx, frame in enumerate(frames):
        draw_detections(frame, detections[frame_idx])
        cv2.imshow('detection', frame)
        if cv2.waitKey(debug_frame_delay) & 0xFF == ord('q'):
            break
    cv2.destroyWindow('detection')


def draw_detections(frame, detections):
    for point in detections:
        cv2.circle(img=frame, center=point, radius=4, color=COLOR_TRACK_FOUND, thickness=1, lineType=cv2.LINE_AA)
