import logging
import sys

import cv2

from crowdstream import config
from crowdstream.services.devices import DeviceService
from crowdstream.utils import tracking
from crowdstream.utils.sequences import generate_sequences
from tests.mocks.CameraServiceMock import CameraServiceMock
from tests.mocks.NetworkServiceMock import NetworkServiceMock

logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)


def test_sequence_generator():
    num_sequences = 10
    prefix = "101"
    streak_limit = 2
    length = 30
    sequences = generate_sequences(num_sequences, length=length, prefix=prefix,
                                   streak_limit=streak_limit)
    assert num_sequences == len(sequences)
    for seq in sequences:
        assert prefix == seq[0:3]
        assert length == len(seq)
        assert 0 == seq.count("000")
        assert 0 == seq.count("111")


def test__record_from_video():
    service = DeviceService(None, None)
    # noinspection PyArgumentList
    cap = cv2.VideoCapture(r"../testdata/1.mp4")
    service._record(cap, 0, r"../test.avi")


def test__record_from_camera():
    service = DeviceService(None, None)
    # noinspection PyArgumentList
    cap = cv2.VideoCapture(0)
    service._record(cap, 5, r"../test.avi")


def test__send_and_capture_sequence():
    # noinspection PyArgumentList
    cap = cv2.VideoCapture(r"../testdata/video/test-(10x10).avi")
    camera_service = CameraServiceMock(cap, (10, 10))
    network_service = NetworkServiceMock(100)
    service = DeviceService(network_service, camera_service)
    service._send_and_capture_sequence(cap, {})


def test__locate_devices():
    # noinspection PyArgumentList
    # cap = cv2.VideoCapture(r"../testdata/video/test-(10x10).avi")
    # DeviceService.DEBUG = True
    # DeviceService.DEBUG_FRAME_DELAY = 10
    camera_service = CameraServiceMock(device_shape=(5, 5))
    network_service = NetworkServiceMock(25)
    service = DeviceService(network_service, camera_service)
    camera_service.plug_camera_mock(service)
    service.position_devices()
    devices = service.devices


def _create_timestamps(num, fps):
    timestamps = []
    for i in range(num):
        timestamps.append(i * 1 / fps)
    return timestamps


def test__track_devices__temp():
    # noinspection PyArgumentList
    filename = rf"../../testdata/video/test__process_sequencing__random_generated-20x20"
    cap = cv2.VideoCapture(rf"{filename}.mp4")
    from tests.test_utils.file import read_obj
    timestamps = read_obj(rf"{filename}-timestamps.txt")
    sequences = read_obj(rf"{filename}-sequences.txt")
    tracking.DEBUG = False
    config.MAP_VISIBLE_FIRST = False
    service = DeviceService(None, None)
    service.DEBUG = False
    service.DEBUG_FRAME_DELAY = 10
    tracks = service._track_devices(cap, timestamps)
    _print_results(sequences, tracks)


def test__track_devices__random_generated():
    print("\n")
    from crowdstream.utils import tracking
    tracking.DEBUG = False
    device_shape = (20, 20)
    num_devices = device_shape[0] * device_shape[1]
    sequence_length = 24
    sequences = generate_sequences(num_devices, sequence_length)
    from tests.test_utils import test_generator
    filename = rf"../../testdata/video/test__process_sequencing__random_generated-{device_shape[0]}x{device_shape[1]}"
    timestamps = test_generator.generate_video(devices_shape=device_shape, path=filename, sequences=sequences,
                                               movement=2, max_start_delay=10, max_movement_distance=3)
    # noinspection PyArgumentList
    cap = cv2.VideoCapture(rf"{filename}.mp4")
    service = DeviceService(None, None)
    service.DEBUG = False
    service.DEBUG_FRAME_DELAY = 10
    tracks = service._track_devices(cap, timestamps)
    # assert len(tracks) == num_devices
    _print_results(sequences, tracks)


def _print_results(sequences, tracks):
    sequence_length = len(sequences[0])
    missed_devices = []
    mapped_devices = 0
    total_devices = len(sequences)
    for track in tracks:
        found = False
        for sequence in sequences:
            if track.get_sequence(sequence_length) == sequence:
                found = True
                sequences.remove(sequence)
                break
        if found:
            mapped_devices += 1
        else:
            missed_devices.append(track)
    print(f"Discovered: {len(tracks)}")
    print(f"Missed {len(missed_devices)} tracks")
    for track in missed_devices:
        print(f"{track.get_sequence(sequence_length)}, {len(track.get_sequence())}")
        print(f"states: {len(track.history)} {track.history}")
    print(f"Missed {len(sequences)} sequences")
    for sequence in sequences:
        print(f"{sequence}, {len(sequence)}")
    print(f"mapped {mapped_devices} of {total_devices} devices")
