import logging
import sys

import cv2
import numpy as np

from crowdstream import config, CrowdstreamServer
from crowdstream.services.devices import DeviceService
from tests.mocks.CameraServiceMock import CameraServiceMock
from tests.mocks.NetworkServiceMock import NetworkServiceMock
from tests.test_utils import generate_ips
from tests.test_utils.file import read_sequences

logging.basicConfig(stream=sys.stderr, level=logging.INFO)


def test__mock_sequence_and_clients():
    server = CrowdstreamServer()
    camera_service = CameraServiceMock(None, (20, 20))
    network_service = NetworkServiceMock(camera_service.device_shape[0] * camera_service.device_shape[1],
                                         server.data_received)
    device_service = DeviceService(network_service, camera_service)
    network_service.mock(device_service)
    camera_service.plug_camera_mock(device_service)
    server._device_service = device_service
    server._network_service = network_service
    server.start()


def test_server():
    camera_service = CameraServiceMock(None)
    network_service = NetworkServiceMock((20, 20), camera_service)
    device_service = DeviceService(network_service, camera_service)
    server = CrowdstreamServer.CrowdstreamServer()
    server.device_service = device_service
    server.start()
    # noinspection PyArgumentList
    # device_service.project_video(cv2.VideoCapture(0))


def test_device_service():
    camera_service = CameraServiceMock(None)
    network_service = NetworkServiceMock((20, 20), camera_service)
    device_service = DeviceService(network_service, camera_service)
    device_service._locate_devices()
    # noinspection PyArgumentList
    device_service.project_video(cv2.VideoCapture(0))


def test_sequencing():
    manager = _setup_manager()
    frame = np.zeros((config.CAMERA_DIMENSIONS[1], config.CAMERA_DIMENSIONS[0], 3), dtype='uint8')
    for device in manager.devices:
        cv2.putText(frame, str(device.ip), (device.position[0], device.position[1]),
                    cv2.FONT_HERSHEY_COMPLEX_SMALL, 0.5, (255, 255, 255))
    cv2.imshow('result', frame)
    cv2.waitKey(0)


def test_image_streaming():
    manager = _setup_manager()
    source: np.ndarray = cv2.imread(r'..\testdata\pattern\random-color-grid-6.jpg')
    manager.project_image(source)
    cv2.waitKey(0)


def test_video_streaming():
    manager = _setup_manager()
    # noinspection PyArgumentList
    cap: cv2.VideoCapture = cv2.VideoCapture(r'../../1.mp4')
    manager.project_video(cap)


def test_video_load(file: str):
    sequences = read_sequences(file)
    ips = generate_ips(len(sequences))
    manager = DeviceService(None, None)
    manager.sequences = sequences
    # noinspection PyArgumentList
    cap = cv2.VideoCapture(rf"..\testdata\{file}.avi")  # type: cv2.VideoCapture
    manager._track_devices(cap)


def _setup_manager():
    camera_service = CameraServiceMock(None)
    network_service = NetworkServiceMock((20, 20), camera_service, None)
    network_service.mock_connections = True
    device_service = DeviceService(network_service, camera_service)
    device_service._locate_devices()
    return device_service
