import time
from typing import List

import cv2

from crowdstream import config
from crowdstream.services import devices
from crowdstream.services.camera import CameraService
from tests.test_utils import test_generator


class CameraServiceMock(CameraService):
    def __init__(self, capture: cv2.VideoCapture = None, device_shape=None):
        super().__init__()
        self.max_start_delay = 0
        self.max_movement_distance = 0
        self.device_size_range = (3, 4)
        self.device_color_variation = 50
        self.device_shape = device_shape
        self.symbol_frequency = config.SYMBOL_FREQUENCY
        self.fps = 30
        self.movement = 0
        self.capture = capture
        self.output_shape = config.CAMERA_DIMENSIONS
        self.sequences = None
        self.device_service = None

    def plug_camera_mock(self, device_service: devices):
        self.device_service = device_service
        device_service.default_warm_up_frames = 0
        func = device_service._generate_sequences

        from functools import wraps

        @wraps(func)
        def capture_sequences(*args, **kwargs):
            self.sequences = func(*args, **kwargs)
            return self.sequences

        device_service._generate_sequences = capture_sequences
        return self

    def get_camera(self) -> cv2.VideoCapture:
        if self.capture:
            return self.capture
        filename = self.generate_simulation(self.sequences)
        # noinspection PyArgumentList
        return cv2.VideoCapture(f"{filename}.mp4")

    def generate_simulation(self, sequences: List[str]):
        devices_x, devices_y = self.device_shape
        filename = rf"..\simulated\sim-({devices_x}x{devices_y})-{time.time()}"
        test_generator.generate_video(devices_shape=self.device_shape, path=f"{filename}",
                                      sequences=sequences,
                                      fps=self.fps, movement=self.movement,
                                      symbol_frequency=self.symbol_frequency,
                                      device_color_variation=self.device_color_variation,
                                      device_size_range=self.device_size_range,
                                      output_shape=self.output_shape,
                                      max_movement_distance=self.max_movement_distance,
                                      max_start_delay=self.max_start_delay)

        return filename
