from typing import Tuple, Dict

import cv2

import crowdstream.utils.video
from crowdstream import config
from crowdstream.services import devices
from crowdstream.services.network import NetworkService


class NetworkServiceMock(NetworkService):
    def __init__(self, connected_devices: int, callback=None):
        super().__init__(callback)
        self.connected_devices = connected_devices
        for i in range(self.connected_devices):
            self._connections[f"mock:ip:{i}"] = None
        self.device_service: devices = None
        self.output = None
        self.client = None

    def send_color_to_ip(self, color: Tuple[int, int, int], ip: str):
        device = [device for device in self.device_service.devices if device.ip == ip][0]
        cv2.circle(self.output, device.position, config.TEST_DEFAULT_DEVICE_SIZE, color, -1)

    def send_sequences(self, symbol_frequency, sequence_map: Dict[str, str]):
        pass

    def connection_made(self, protocol):
        pass

    def connection_lost(self, protocol):
        pass

    def _get_device(self, ip):
        return [device for device in self.device_service.devices if device.ip == ip][0]

    def mock(self, device_service: devices):
        self.device_service = device_service
        device_service._ips = self._connections
        func = device_service._send_colors_to_devices

        def mock_send_colors(*args, **kwargs):
            self.output = crowdstream.utils.video.create_empty_frame(config.CAMERA_DIMENSIONS[0],
                                                                     config.CAMERA_DIMENSIONS[1])
            func(*args, **kwargs)
            cv2.imshow('output', self.output)
            cv2.waitKey(1)

        device_service._send_colors_to_devices = mock_send_colors
        return self
