import timeit

import cv2
import matplotlib.pyplot as plt
from pytest import skip

from crowdstream.utils import detection, tracking
from tests.test_utils import TESTDATA_DIR, test_generator, file


def test__detection_performance__real_video():
    cap: cv2.VideoCapture = cv2.VideoCapture(rf"{TESTDATA_DIR}/tracking/1.mp4")
    frames = []
    while cap.isOpened():
        success, frame = cap.read()
        if not success:
            break
        frames.append(frame)
    num_frames = len(frames)
    hz_values = []
    detection_values = []
    for threshold in range(255, 100, -1):
        detections = 0
        start = timeit.default_timer()
        for frame in frames:
            detections = detections + len(detection.detect_devices(frame, threshold))
        end = timeit.default_timer()
        duration = end - start
        hz = num_frames / duration
        hz_values.append(hz)
        detection_values.append(detections / num_frames)

    plt.plot(detection_values, hz_values)
    plt.ylabel("Hz")
    plt.xlabel("Detected objects")
    plt.show()

    print()
    print(f"frames: {num_frames}")
    print(f"time: {duration}")
    print(f"detections: {detections}")
    print(f"hz: {num_frames/duration}")


def test__tracking_performance__generated_video():
    filename = rf'{TESTDATA_DIR}/performance/tracking'
    cap = cv2.VideoCapture(f"{filename}.mp4")
    frames = []
    while cap.isOpened():
        success, frame = cap.read()
        if not success:
            break
        frames.append(frame)
    timestamps = file.read_obj(rf"{filename}-timestamps.txt")
    num_frames = len(timestamps)
    hz_values = []
    detection_values = []
    for threshold in range(250, 220, -5):
        positions = []
        for frame in frames:
            positions.append(detection.detect_devices(frame, threshold))
        start = timeit.default_timer()
        found_tracks = len(tracking.perform_tracking(positions, timestamps))
        end = timeit.default_timer()
        duration = end - start
        hz = num_frames / duration
        hz_values.append(hz)
        detection_values.append(found_tracks)

    plt.plot(detection_values, hz_values)
    plt.ylabel("Hz")
    plt.xlabel("Detected objects")
    plt.show()

    print()
    print(f"frames: {num_frames}")
    print(f"time: {duration}")
    print(f"detections: {found_tracks}")
    print(f"hz: {num_frames/duration}")


@skip("Only used to generate a video for the tracking_performance test")
def test__generate_test_video():
    filename = rf'{TESTDATA_DIR}/performance/tracking'
    test_generator.generate_video((20, 20), filename, movement=0, device_size_range=(3, 3), max_movement_distance=0,
                                  device_color_variation=60)
