import logging
import msvcrt
import os
import sys
import win32api

import cv2
# noinspection PyProtectedMember
from twisted.internet import _pollingfile, reactor
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet.protocol import connectionDone
from twisted.protocols.basic import LineOnlyReceiver

from crowdstream.utils import create_empty_frame
from crowdstream.utils.types import Color

WIDTH = 5
HEIGHT = 5

logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
logger = logging.getLogger(__name__)

DEBUG = True


# noinspection PyProtectedMember
class TestClient(LineOnlyReceiver, _pollingfile._PollingTimer):
    def __init__(self):
        super().__init__(reactor)
        for stdfd in range(0, 1, 2):
            msvcrt.setmode(stdfd, os.O_BINARY)

        _hstdin = win32api.GetStdHandle(win32api.STD_INPUT_HANDLE)

        self.stdin = _pollingfile._PollableReadPipe(
            _hstdin, self.input, self.read_connection_lost)

        self._addPollableResource(self.stdin)
        self.frame = create_empty_frame(WIDTH, HEIGHT)

    def lineReceived(self, line: bytes):
        if not DEBUG:
            return
        msg = line.decode()
        tokens = msg.split(',')
        if tokens[0] == 'c':
            self.handle_color((int(tokens[1]), int(tokens[2]), int(tokens[3])))
        elif tokens[0] == 's':
            self.handle_sequence(int(tokens[1]), tokens[2])

    def handle_color(self, color: Color):
        r, g, b = color
        cv2.circle(self.frame, (5, 5), 10, (b, g, r), -1)
        cv2.imshow('received', self.frame)
        cv2.waitKey(1)

    def handle_sequence(self, symbol_frequency: int, sequence: str):
        delay = int(1000 / symbol_frequency)
        # fps = 30
        # frames_per_symbol = int(fps / symbol_frequency)
        from crowdstream import config
        # noinspection PyArgumentList
        # writer = cv2.VideoWriter(r"../../sequence.avi", cv2.VideoWriter_fourcc(*"MJPG"), fps, config.CAMERA_DIMENSIONS, True)
        for symbol in sequence:
            frame = create_empty_frame(config.CAMERA_DIMENSIONS[0], config.CAMERA_DIMENSIONS[1])
            color = (255, 255, 255) if symbol == '1' else (0, 0, 0)
            cv2.circle(self.frame, (5, 5), 10, color, -1)
            cv2.circle(frame, (400, 400), 10, color, -1)
            # for _ in range(frames_per_symbol):
            #     writer.write(frame)
            cv2.imshow('sequence', self.frame)
            cv2.waitKey(delay)
            # writer.release()

    def input(self, data: bytes):
        self.sendLine(data.strip())

    @staticmethod
    def read_connection_lost():
        logger.info('read_connection_lost')

    def connectionLost(self, reason=connectionDone):
        logger.info("connection lost")

    def connectionMade(self):
        logger.info("connected")
        self.sendLine("pos 0 0".encode())
        logger.info("position sent")


def main():
    test_client = TestClient()
    endpoint = TCP4ClientEndpoint(reactor, "127.0.0.1", 8000)
    connectProtocol(endpoint, test_client)
    # noinspection PyUnresolvedReferences
    reactor.run()


if __name__ == '__main__':
    main()
