import time

import cv2
import numpy as np
from scipy.spatial import distance

from crowdstream.utils import create_empty_frame
from tests.test_utils import test_generator


def test_get_point_in_radius():
    width, height = 500, 500
    center = int(width / 2), int(height / 2)
    frame = create_empty_frame(500, 500)
    cv2.circle(frame, center, 5, (0, 0, 255), -1)
    last_point = center
    for _ in range(1000):
        new_point = test_generator._get_point_in_radius(last_point, 20)
        distance_from_origin = None
        while not distance_from_origin or distance_from_origin > 100:
            new_point = test_generator._get_point_in_radius(last_point, 20)
            distance_from_origin = distance.cdist(np.array([center]), np.array([new_point]))[0]
        cv2.circle(frame, new_point, 1, (255, 255, 255), -1)
        last_point = new_point
    cv2.imshow('test', frame)
    cv2.waitKey()


def test_get_point_in_radius_():
    pt = (100, 100)
    radius = 50
    for _ in range(100000):
        new_point = test_generator._get_point_in_radius(pt, radius)
        dist = distance.cdist(np.array([pt]), np.array([new_point]))[0]
        assert int(dist) <= radius


def test_generate_video():
    devices_x, devices_y = 20, 20
    width, height = 1080, 720
    filename = rf"..\testdata\video\test-({devices_x}x{devices_y})-{time.time()}"
    # filename = rf"..\testdata\video\test-({devices_x}-{devices_y})"
    # sequences = _generate_sequences(devices_x * devices_y, 32, prefix="101", streak_limit=3)
    sequence = "1" * 48
    sequences = []
    for _ in range(devices_y * devices_x):
        sequences.append(sequence)
    test_generator.generate_video(devices_shape=(devices_x, devices_y), path=f"{filename}", sequences=sequences,
                                  fps=30, movement=1, max_movement_distance=20, symbol_frequency=10,
                                  device_color_variation=50,
                                  device_size_range=(2, 5), output_shape=(width, height))
